#include "stdafx.h"
#include "MarkerDetector.h"

int RegisterPattern( const vector<PATTERN_DATA> &pattern_list )
{
	int n = pattern_list.size();
	for ( int i=0; i<n; i++ ) {
		int id = arLoadPatt( pattern_list[i].filename.c_str() );
		if (id == -1) return -1;
		if ( !(pattern_list[i].enable) ) arDeactivatePatt(i);
	}
	return n;
}

int LoadPatternList( const char *filename, vector<PATTERN_DATA> &pattList )
{	
	ifstream fin(filename);
	if (!fin) {
		cout << filename << " can not open.\n";
		return -1;
	}

	pattList.clear();

	string word;
	string enable;
	PATTERN_DATA patt;

	while (fin >> word) {
		if (word=="[Pattern]") {
			fin >> patt.name >> patt.filename >> patt.width	>> enable;
			patt.enable = (enable=="use" ? true : false);
			patt.id = pattList.size();
			pattList.push_back(patt);
		}
	}	
	fin.close();

	return 0;
}

static void GetDataFromARMarkerInfo3D( ARMarkerInfo* marker, AR_OBJECT &object)
{
	object.area = marker->area;
	object.id = marker->id;
	object.cf = marker->cf;
	object.px = marker->pos[0];
	object.py = marker->pos[1];

	// order correction
	// LeftTop -> RightTop -> RightBottom -> LeftBottom
	for (int j=0; j<4; j++) {
		int c = (j + 4 - marker->dir)%4;
		object.vertex[j][0] = marker->vertex[c][0];
		object.vertex[j][1] = marker->vertex[c][1];
	}

	// normalized vector
	double vec[2];
	vec[0] = (object.vertex[0][0] - object.px ) + (object.vertex[1][0] - object.px);
	vec[1] = (object.vertex[0][1] - object.py ) + (object.vertex[1][1] - object.py);

	double norm = sqrt( vec[0]*vec[0] + vec[1]*vec[1] );
	object.dx = vec[0]/norm;
	object.dy = vec[1]/norm;

	// Get 3D information
	double center[2] = {0.0,0.0};
	double trans[3][4];
	arGetTransMat( marker, center, object.width, trans );
	object.X = trans[0][3];
	object.Y = trans[1][3];
	object.Z = trans[2][3];
	argConvGlpara( trans, object.gltrans );
}


int DetectMarker( ARUint8 *image, int thresh, const vector<PATTERN_DATA> &patt_list, vector<AR_OBJECT> &objects )
{
	ARMarkerInfo	*marker_info;
	int				marker_num;
	double			center[2] = {0.0, 0.0};

	if ( image== NULL) return -1;

	//objects.clear();
	AR_OBJECT one_object;

	if ( arDetectMarkerLite( image, thresh, &marker_info, &marker_num ) ) {
		// use arDetectMarker"Lite"!!
		printf("error in arDetectMaker\n");
		return -1;
	}

	// multi same markers can be exist in the image.
	for (int i=0; i<marker_num; i++) {
		int k = -1;
		for (unsigned int j=0; j<patt_list.size(); j++) {
			if ( marker_info[i].id == patt_list[j].id ) {
				k = j;
				break;
			}
		}
		if (k!=-1 && marker_info[i].cf>=0.6 ) {
			one_object.width = patt_list[k].width;
			one_object.name  = patt_list[k].name;
			GetDataFromARMarkerInfo3D( &marker_info[i], one_object );
			objects.push_back( one_object );
		}
	}

	return 0;
}
